(function(angular){

    var app = angular.module('new-course');

    app.controller('CourseEditController',
        ['$scope', '$window', '$uibModal', '$http', '$q', 'Course',  'CourseAuthor', 'Lesson', 'SimpleLesson', '$filter', 'youtubePlayerApi', 'VideoData', 'FormUpload', 'uibDateParser', 'SetLanguage', 'UserLocalStorage', 'Upload', 'Videos','gettextCatalog',
        function($scope, $window, $uibModal, $http , $q, Course,  CourseProfessor, Lesson, SimpleLesson, $filter, youtubePlayerApi, VideoData, FormUpload, uibDateParser, SetLanguage, UserLocalStorage, Upload, Videos, gettextCatalog) {

            $scope.pt_checked = false;
            $scope.en_checked = false;
            $scope.es_checked = false;

            $scope.editingCourseLanguage = UserLocalStorage.get('editingCourseLanguage') || UserLocalStorage.get('currentLanguage');

            $scope.modal_lang = UserLocalStorage.get('currentLanguage');

            $scope.errors = {};
            var httpErrors = {
                '400': gettextCatalog.getString('Fields were not filled in correctly.'),
                '403': gettextCatalog.getString('You do not have permission to view content on this page.'),
                '404': gettextCatalog.getString('This course does not exist!')
            };

            $scope.setLanguage = function(language) {
                UserLocalStorage.set('editingCourseLanguage', language);
                SetLanguage(language).then(function() {
                    $window.location.reload();
                });
            };

            $scope.course_id = parseInt($window.course_id, 10);
            $scope.courseProfessors = [];
            $scope.lessons = [];

            Course.get({id: $scope.course_id}, function(course){
                if(course.intro_video) {
                    youtubePlayerApi.videoId = course.intro_video.youtube_id;
                    $scope.link_video = "//www.youtube.com/embed/" + course.intro_video.youtube_id + "?rel=0&showinfo=0&autohide=1&theme=light&wmode=opaque";
                }
                document.title = `${gettextCatalog.getString('Course')}: ${course.name}`;
                $scope.addThumb = !course.thumbnail_url;
                $scope.addHomeThumb = !course.home_thumbnail_url;
                course.start_date = uibDateParser.parse(course.start_date, "yyyy-MM-dd");
                $scope.course = course;
                $scope.ct_id = course.ct_id;

                (course.lang || []).forEach(function(l) {
                    if (l === 'en') {
                        $scope.en_checked = true;
                    }
                    if (l === 'pt-br') {
                        $scope.pt_checked = true;
                    }
                    if (l === 'es') {
                        $scope.es_checked = true;
                    }
                });
            });

            $scope.changeVideo = function() {
                $scope.course.intro_video.youtube_id = '';
                $scope.removeVideo();
            }

            $scope.uploadVideo = function(file) {
                if ($scope.course && $scope.course.intro_video) {
                    let video_id = $scope.course.intro_video.id;
                    let youtube_id = '';

                    if (file) {
                        let name = file.name;

                        if (video_id) {
                            youtube_id = $scope.course.intro_video.youtube_id;
                            Videos.upload(file, youtube_id, name, video_id).then(function(response) {
                                $scope.course.intro_video.file = response.data;
                            }).catch(function(error) {
                                $scope.error_messages.push(
                                    gettextCatalog.getString('There was an error saving the video.')
                                );
                                console.error(error);
                            });
                        } else {
                            $scope.course.save().then(function(){
                                Course.get({id: $scope.course_id}, function(course){
                                    video_id = course.intro_video.id;
                                    youtube_id = course.intro_video.id;
                                    Videos.upload(file, youtube_id, name, video_id).then(function(response) {
                                        $scope.course.intro_video.file = response.data;
                                    }).catch(function(error) {
                                        $scope.error_messages.push(
                                            gettextCatalog.getString('There was an error saving the video.')
                                        );
                                        console.error(error);
                                    });
                                });
                            });
                        }
                    }
                }
            };

            $scope.getVideoName = function() {
                video_name = '';
                if($scope.course.intro_video.file){
                    if (typeof($scope.course.intro_video.file) === 'string') {
                        video_name = $scope.course.intro_video.file;
                    }
                    else {
                        video_name = $scope.course.intro_video.file.file;
                    }
                }
                video_name = video_name.split('/')[5];
                return video_name;
            }

            $scope.selectVideo = function(file) {
                if (file) {
                    $scope.course.intro_video.file = file;
                }
            }

            $scope.removeVideo = function() {
                if (window.confirm(gettextCatalog.getString("Do you really want to delete this video?"))) {
                  Videos.update({ id: $scope.course.intro_video.id, file: null }).$promise.then(function(){
                      $scope.course.intro_video.file = null;
                      //$scope.course.save();
                  });
                }
            }

            $scope.lessons = SimpleLesson.query({
              'course__id': $scope.course_id
            });

            $scope.courseProfessors = CourseProfessor.query({
              course: $scope.course_id
            });

            var player;
            $scope.playerReady = false;
            youtubePlayerApi.loadPlayer().then(function(p){
                player = p;
                player.cueVideoById(youtubePlayerApi.videoId);
                $scope.playerReady = true;
            });

            $scope.$watch('course.intro_video.youtube_id', function(vid, oldVid){
                if(!vid || vid === oldVid) return;
                if(player) player.cueVideoById(vid);
                VideoData.load(vid).then(function(data){
                    if (data.items.length > 0)
                        $scope.course.intro_video.name = data.items[0].snippet.title;
                });
            });

            $scope.setCurrentUnitVideo = function() {
                //
                // support pasting both long and short urls from youtube
                // eg. http://youtu.be/8uj7YSqby7s
                //
                var complete_url = /^.*(youtu\.be\/|v\/|u\/\w\/|embed\/|watch\?v=|\&v=)([^#\&\?]*).*/;
                var result = complete_url.exec($scope.youtube_id);
                if (result && result[2].length == 11) {
                    // If this is the first time a intro_video is set, prepare the variable
                    if($scope.course.intro_video === 'undefined' || $scope.course.intro_video === null)
                        $scope.course.intro_video = {};
                    $scope.course.intro_video.youtube_id = result[2];
                    $scope.link_video = "//www.youtube.com/embed/" + $scope.course.intro_video.youtube_id + "?rel=0&showinfo=0&autohide=1&theme=light&wmode=opaque";
                }
            };

            function showFieldErrors(response) {
                $scope.errors = response.data;
                var messages = [];
                for(var att in response.data) {
                    var message = response.data[att];
                    messages.push(message);
                }
                $scope.alert.error(gettextCatalog.getString("We found some errors!"), messages, true);
            }

            $scope.saveThumb = function() {
                if(! $scope.thumbfile) {
                    return;
                }

                if ($scope.course.id) {
                    var fu = new FormUpload();
                    fu.addField('thumbnail', $scope.thumbfile);
                    // return a new promise that file will be uploaded
                    return fu.sendTo(BASE_API_URL + '/coursethumbs/' + $scope.course.id)
                        .then(function(){
                            $scope.alert.success(gettextCatalog.getString("Updated image"));
                        });
                }
            };

            $scope.saveHomeThumb = function() {
                if(! $scope.home_thumbfile) {
                    return;
                }

                if ($scope.course.id) {
                    var fu = new FormUpload();
                    fu.addField('home_thumbnail', $scope.home_thumbfile);
                    // return a new promise that file will be uploaded
                    return fu.sendTo(BASE_API_URL + '/coursethumbs/' + $scope.course.id)
                        .then(function(){
                            $scope.alert.success(gettextCatalog.getString("Updated image"));
                        });
                }
            };

            $scope.export_course = function() {
                $window.open('/legacy/admin/course/' + $scope.course_id + '/export/', '_blank', '');
            };

            $scope.saveCourse = function() {
                var languages = [];

                if ($scope.en_checked) {
                    languages.push('en')
                }
                if ($scope.pt_checked) {
                    languages.push('pt-br')
                }
                if ($scope.es_checked) {
                    languages.push('es')
                }
                $scope.course.lang = languages;

                if(!$scope.course.hasVideo()){
                    delete $scope.course.intro_video;
                }
                if(!$scope.course.slug){
                    $scope.course.slug = $filter('slugify')($scope.course.name);
                }

                if ($scope.course.start_date) {
                    $scope.course.start_date = $filter('date')($scope.course.start_date, 'yyyy-MM-dd');
                }

                $scope.course.save()
                    .then(function(course){
                        course.start_date = uibDateParser.parse(course.start_date, "yyyy-MM-dd");
                        return $scope.saveThumb();
                    })
                    .then(function(){
                        $scope.alert.success(gettextCatalog.getString('Changes saved successfully.'));
                    })['catch'](showFieldErrors);
            };

            $scope.publishCourse = function() {
                $scope.course.status = 'published';
                $scope.saveCourse();
            };

            $scope.set_course_as_draft = function() {
                let text = 'By making the course a draft, it will no longer be visible to users! Are you sure you want to continue?';
                if (confirm(gettextCatalog.getString(text))){
                    $scope.course.status = 'draft';
                    $scope.saveCourse();
                }
            };

            $scope.deleteCourse = function() {
                if(!confirm(gettextCatalog.getString('Are you sure you want to remove this course?'))) return;

                $scope.course.$delete()
                    .then(function(){
                        document.location.href = '/legacy/admin/courses';
                    });
            };

            $scope.delete_instructor = function(course_author) {
                let text = gettextCatalog.getString('Are you sure you want to remove');
                let last_text = gettextCatalog.getString('from the list of instructors for this course?');
                var confirm_exclude_instructor_msg = `${text} ${course_author.get_name} ${last_text}`;
                if (window.confirm(confirm_exclude_instructor_msg)){
                    course_author.$delete().then(function(){
                        var index = $scope.courseProfessors.indexOf(course_author);
                        $scope.courseProfessors.splice(index, 1);
                        let text = gettextCatalog.getString('has been removed from the list of instructors.');
                        $scope.alert.success(`${course_author.get_name} ${text}`);
                    }, function(){
                        let text = gettextCatalog.getString('Error removing');
                        let last_text = gettextCatalog.getString('from the list of instructors.');
                        $scope.alert.error(`${text} ${course_author.get_name} ${last_text}`);
                    });
                }
            };

            $scope.save_all_instructors = function() {
                var promises_list = [];
                $scope.courseProfessors.forEach(function(course_author) {
                    delete course_author.picture;
                    course_author.$update();
                    promises_list.push(course_author.$promise);
                });
                $q.all([promises_list]).then(function() {
                        $scope.alert.success(gettextCatalog.getString('Instructors position successfully saved.'));
                    }, function() {
                        $scope.alert.error(gettextCatalog.getString('It was not possible to save the position of the instructors!'));
                    }
                );
            };

            $scope.open_professor_modal = function(course_professor) {

                $scope.courseProfessors_original = angular.copy($scope.courseProfessors);
                var modalInstance = $uibModal.open({
                       templateUrl: 'course_professor_modal.html',
                       controller: CourseProfessorModalInstanceCtrl,
                       resolve: {
                           course_professor: function () {
                               return course_professor;
                           }
                       }
                });

                modalInstance.result.then(function (course_professor) {
                    var course_professor_picture_file = course_professor.course_professor_picture_file;
                    var fu = new FormUpload();
                    fu.addField('picture', course_professor_picture_file);

                    if (course_professor.picture !== null)
                        delete course_professor.picture;

                    if (course_professor.id === undefined){
                        course_professor.course = $scope.course_id;
                        course_professor.position = $scope.courseProfessors.length;

                        course_professor.$save({}, function (course_professor){

                            if (course_professor_picture_file){
                                // return a new promise that file will be uploaded
                                fu.sendTo(BASE_API_URL + '/course_professor_picture/' + course_professor.id)
                                    .then(function(response){
                                        course_professor.get_picture_url = response.data.picture;
                                        course_professor.picture = response.data.picture;
                                });
                            }

                            $scope.courseProfessors.push(course_professor);
                            $scope.alert.success(gettextCatalog.getString('Course author successfully updated!'));
                        }, function (){
                            $scope.alert.error(gettextCatalog.getString('Unable to update course author!'));
                        });
                    } else {
                        course_professor.$update({id: course_professor.id}, function (){
                            if (course_professor_picture_file){
                                // return a new promise that file will be uploaded
                                fu.sendTo(BASE_API_URL + '/course_professor_picture/' + course_professor.id)
                                    .then(function(response){
                                        course_professor.get_picture_url = response.data.picture;
                                        course_professor.picture = response.data.picture;
                                });
                            }
                            $scope.alert.success(gettextCatalog.getString('Course author successfully updated!'));
                        }, function (){
                            $scope.alert.error(gettextCatalog.getString('Unable to update course author!'));
                        });
                    }
                },
                function() {
                    $scope.courseProfessors = angular.copy($scope.courseProfessors_original);
                });
            };

            var CourseProfessorModalInstanceCtrl = function($scope, $uibModalInstance, course_professor) {
                if (course_professor === undefined) {
                    course_professor = new CourseProfessor();
                    $scope.linked_with_user = null;
                }

                $scope.course_professor = course_professor;

                $scope.link_user_on = function () {
                    $scope.linked_with_user = true;
                    if (course_professor.user) {
                        if (!course_professor.name && course_professor.user_info.name) {
                            $scope.name_from_user_profile = true;
                        }

                        if (!course_professor.biography && course_professor.user_info.biography) {
                            $scope.biography_from_user_profile = true;
                        }

                        if (!course_professor.picture && course_professor.user_info.image) {
                            $scope.picture_from_user_profile = true;
                        }
                    }
                };

                if (course_professor.user_info) {
                    $scope.link_user_on();
                } else if ($scope.course_professor.id) {
                    $scope.linked_with_user = false;
                }


                $scope.remove_user_link = function () {
                    $scope.linked_with_user = false;
                    $scope.name_from_user_profile = false;
                    $scope.biography_from_user_profile = false;
                    $scope.picture_from_user_profile = false;
                    $scope.show_form = true;
                };

                $scope.cancel = function () {
                    $uibModalInstance.dismiss();
                };

                $scope.on_select_professor = function(model) {
                    $scope.course_professor.user = model.id;
                    $scope.course_professor.user_info = model;
                    $scope.link_user_on();
                    $scope.show_form = true;
                };

                $scope.remove_professor = function() {
                    $scope.course_professor.user = null;
                    $scope.course_professor.user_info = null;
                    $scope.name_from_user_profile = false;
                    $scope.biography_from_user_profile = false;
                    $scope.picture_from_user_profile = false;
                };

                $scope.save_course_professors = function() {

                    if ($scope.linked_with_user) {
                        if ($scope.name_from_user_profile) {
                            $scope.course_professor.name = null;
                        }
                        if ($scope.biography_from_user_profile) {
                            $scope.course_professor.biography = null;
                        }
                        if ($scope.picture_from_user_profile) {
                            $scope.course_professor.picture = null;
                            $scope.course_professor.get_picture_url = null;
                        }
                    } else {
                        $scope.course_professor.user = null;
                    }

                    if ($scope.course_professor_picture_file && !$scope.picture_from_user_profile) {
                        $scope.course_professor.course_professor_picture_file = $scope.course_professor_picture_file;
                    }
                    $uibModalInstance.close($scope.course_professor);
                    $scope.course_professor = undefined;
                };

                $scope.getUsers = function(val) {
                    return $http.get(BASE_API_URL + '/user_search', {
                        params: {
                          name: val,
                          sensor: false
                        }
                    }).then(function(res){
                        var professors_found = [];
                        angular.forEach(res.data, function(item){
                            var formated_name = '';
                            if (item.first_name)
                                formated_name += item.first_name;
                            if (item.last_name)
                                formated_name = formated_name + ' ' + item.last_name;
                            if (formated_name)
                                formated_name = formated_name + ' - ';
                            formated_name += item.username;
                            if (item.email)
                                formated_name = formated_name + ' - ' + item.email;
                            item.formated_name = formated_name;
                            professors_found.push(item);
                        });
                        return professors_found;
                    });
                };
            };

            $scope.saveLesson = function(lesson) {
                return lesson.saveOrUpdate()
                    .then(() => {
                        $scope.saveThumbLesson()
                    }).then(() => {
                        $scope.alert.success(gettextCatalog.getString('Module updated successfully'));
                    }).catch(() => {
                        $scope.alert.error(gettextCatalog.getString('Unable to save module'));
                    });
            };

            $scope.deleteLesson = function(lesson) {
                let msg = gettextCatalog.getString('Are you sure you want to delete this module?');
                if(!window.confirm(`${msg} ${lesson.name}`)) return;

                return lesson.$delete().then(function(){
                    var filter = function(l) { return l.id !== lesson.id; };
                    $scope.lessons = $scope.lessons.filter(filter);
                    let msg_deleted = gettextCatalog.getString('was deleted');
                    $scope.alert.success(`${lesson.name} ${msg_deleted}`);
                });
            };

            $scope.repositionLessons = function() {
                $scope.lessons.forEach(function(lesson, i){
                    lesson.position = i;
                });
            };

            $scope.repositionInstructors = function() {
                $scope.courseProfessors.forEach(function(p, i){
                    p.position = i;
                });
            };

            $scope.saveAllLessons = function() {
                var i = 0;
                function __saveLessons() {
                    if(i < $scope.lessons.length) {
                        return $scope.lessons[i++]
                                     .saveOrUpdate()
                                     .then(__saveLessons);
                    }
                }

                $scope.alert.warn(gettextCatalog.getString('Updating modules'));

                __saveLessons()
                    .then(function(){
                        $scope.alert.success(gettextCatalog.getString('Modules have been updated'));
                    })['catch'](function(){
                        $scope.alert.error(gettextCatalog.getString('Some problem prevented modules from updatin'));
                    });
            };

        }
    ]);

})(window.angular);
