(function(angular){
    'use strict';

    var module = angular.module('edit_class.controllers', []);

    // Service to share data across the controllers
    module.factory('classe_list', function() {
        return {
            classe : []
        };
    });

    module.controller('ClassesListController', [
        '$scope',
        '$location',
        'CurrentUser',
        'BasicClass',
        'Contracts',
        'Course',
        'CourseProfessor',
        'TranslateSlug',
        function(
            $scope,
            $location,
            CurrentUser,
            Class,
            Contracts,
            Course,
            CourseProfessor,
            TranslateSlug
        ) {
            var url_path = $location.absUrl().split('/');
            var course_slug = url_path.slice(-3)[0];

            $scope.classes = {};
            $scope.current_user_course_role = '';
            $scope.current_user = CurrentUser;

            var course;

            TranslateSlug(course_slug).then(function(resp) {
                let translated_slug = resp.data.slug;
                Course.query({slug: translated_slug}).$promise.then(function(response) {
                    course = response[0];
                    $scope.classes.all = Class.query({course: course.id});
                    $scope.classes.filtered = $scope.classes.all;

                    CourseProfessor.query({
                        course: $scope.course_id,
                        user: $scope.current_user.id
                    }, function(course_professor) {
                        if (Array.isArray(course_professor) && course_professor.length) {
                            $scope.current_user_course_role = course_professor[0].role;
                        }
                    });
                });
            });

            $scope.contracts = Contracts.query({'simple' : true});
            $scope.filters = {
                contract: ''
            };

            $scope.$watch('filters.contract', function(newContract, oldContract) {
                if (!newContract)
                    $scope.classes.filtered = $scope.classes.all;
                else if (newContract)
                    $scope.classes.filtered = $scope.classes.all.filter(function(klass) {
                        if (klass.contract)
                            return klass.contract.id === newContract.id;
                        return false;
                    });
            });
        }
    ]);

    module.controller('EditClassController', [
        '$scope',
        '$window',
        'CurrentUser',
        'Class',
        'StudentSearch',
        'Contracts',
        'CourseCertification',
        'CertificateTemplate',
        'CourseProfessor',
        'UserLocalStorage',
        'gettextCatalog',
        function(
            $scope,
            $window,
            CurrentUser,
            Class,
            StudentSearch,
            Contracts,
            CourseCertification,
            CertificateTemplate,
            CourseProfessor,
            UserLocalStorage,
            gettextCatalog
        ) {

            UserLocalStorage.set('currentLanguage', UserLocalStorage.get('currentLanguage'));

            $scope.errors = {};
            var httpErrors = {
                '400': gettextCatalog.getString('Fields were not filled in correctly.'),
                '403': gettextCatalog.getString('You do not have permission to view content on this page.'),
                '404': gettextCatalog.getString('This class does not exist!'),
                '405': gettextCatalog.getString('Something seems to have gone wrong!'),
            };

            $scope.current_user_course_role = '';
            $scope.current_user = CurrentUser;
            $scope.can_edit_this_class = CurrentUser.is_superuser;

            $scope.templates_selected = {};

            $scope.class_id = parseInt($window.class_id, 10);
            $scope.course_id = parseInt($window.course_id, 10);
            $scope.classe = Class.get({id: $scope.class_id}, function(classe) {
                classe.students_details = classe.students_details.sort((a,b) => {
                        // sort by name
                        if (a.name === null) {
                            return 1;
                        }
                        if (b.name === null) {
                            return -1;
                        }

                        const nameA = a.name.toUpperCase();
                        const nameB = b.name.toUpperCase();
                        return (nameA < nameB) ? -1 : 1;
                    })
                
                // TODO translate
                document.title = 'Turma: {0}'.format(classe.name);

                if (classe.certificate_template) {
                    CertificateTemplate.get({
                        id: classe.certificate_template,
                    }, function(template) {
                        $scope.templates_selected.certificate = template;
                    });
                }

                if (classe.receipt_template) {
                    CertificateTemplate.get({
                        id: classe.receipt_template,
                    }, function(template) {
                        $scope.templates_selected.receipt = template;
                    });
                }
            });

            CourseProfessor.query({
                course: $scope.course_id,
                user: $scope.current_user.id
            }, function(course_professor) {
                // Remove useless classes in ui-select
                let ui_selects = document.querySelectorAll('.ui-select-toggle');
                ui_selects.forEach(function(el) {
                    el.classList.remove('btn');
                    el.classList.remove('btn-default');}
                )
                if (Array.isArray(course_professor) && course_professor.length) {
                    $scope.current_user_course_role = course_professor[0].role;
                    if ($scope.current_user_course_role == 'coordinator' || $scope.current_user_course_role == 'assistant')
                        $scope.can_edit_this_class = true;
                }
            });

            CertificateTemplate.query({
                course: $scope.course_id,
            }, function(templates) {

                $scope.certificate_templates_available = [];
                $scope.receipt_templates_available = [];
                angular.forEach(templates, function(template) {
                    if (template.document_type == 'certificate'){
                        $scope.certificate_templates_available.push(template);
                    } else if (template.document_type == 'receipt') {
                        $scope.receipt_templates_available.push(template);
                    }
                });
            });

            $scope.contracts = Contracts.query({'simple' : true});

            $scope.getUsers = function(val) {
                return new StudentSearch(val, course_id);
            };

            $scope.remove_item = function(index){
                // removing from 'screen' list
                var student_id = $scope.classe.students_details[index].id;
                $scope.classe.students_details.splice(index, 1);

                // remove from real list
                var index_management = $scope.classe.students.indexOf(student_id);
                $scope.classe.students.splice(index_management, 1);
                $scope.save();
            };

            $scope.on_select_student = function(model) {
                $scope.classe.students_details.unshift(model);
                $scope.classe.students.unshift(model.id);
                $scope.asyncSelected = '';
                $scope.save();
            };

            $scope.toggle_class_certificable = function (){
                $scope.save();
            }

            $scope.toggle_certificate = function (index){
                var student = $scope.classe.students_details[index];

                if(student.certificate.exists) {
                    var cc_id = student.certificate.link_hash;
                    var user_id = student.id;
                    var cc = CourseCertification.get({link_hash: cc_id, user: user_id}, function(classe) {
                        if(cc.type === 'receipt') {
                            cc.type = 'certificate';
                            $scope.classe.students_details[index].certificate.type = 'certificate';
                        } else {
                            cc.type = 'receipt';
                            $scope.classe.students_details[index].certificate.type = 'receipt';
                        }
                        cc.$update({link_hash: cc_id, user: user_id});
                    });
                }
            }

            $scope.save = function(){

                if ( $scope.templates_selected.certificate && $scope.templates_selected.certificate.id) {
                    $scope.classe.certificate_template = $scope.templates_selected.certificate.id;
                }
                if ( $scope.templates_selected.receipt && $scope.templates_selected.receipt.id) {
                    $scope.classe.receipt_template = $scope.templates_selected.receipt.id;
                }

                $scope.classe.$update()
                    .then(function(){
                        $scope.alert.success(gettextCatalog.getString('Changes saved successfully.'));

                        // remove pop-up that confirm if user go without save changes
                        window.onbeforeunload = function(){};

                        })['catch'](function(resp){
                            $scope.alert.error(httpErrors[resp.status.toString()]);
                        });
            };

            $scope.delete_class = function() {
                let slug = $scope.classe.course.slug;
                if (confirm(gettextCatalog.getString('Are you sure you want to delete this class') + `: '${$scope.classe.name}' ?`)) {
                    $scope.classe.$delete().then(function(){
                        window.location.href = `/legacy/course/${slug}/classes/`;
                    });
                }
            }
        }
    ]);
})(angular);
