(function() {
    'use strict';

    angular
        .module('classroomParticipationReport')
        .component('classroomParticipationReport', {
            templateUrl: '/classroom-participation-report.template.html',
            controller: ClassroomParticipationReportCtrl,
            bindings: {
                date: '<',
            },
        });

    ClassroomParticipationReportCtrl.$inject = ['$scope', 'ClassroomReport'];

    function ClassroomParticipationReportCtrl ($scope, ClassroomReport) {

        var ctrl = this;

        ctrl.classrooms = [];
        ctrl.filterClassrooms = filterClassrooms;
        ctrl.filteredClassrooms = [];
        ctrl.minimized = false;
        ctrl.search = '';
        ctrl.toggleDetails = toggleDetails;
        ctrl.total = null;

        ctrl.$onChanges = function() {
            fetchClassrooms();
        };

        function fetchClassrooms() {
            var params = { date: ctrl.date };

            ClassroomReport.query(params, (data) => {
                ctrl.classrooms = data;
                filterClassrooms();
            });
        }

        function filterClassrooms() {
            if (ctrl.search) {
                ctrl.filteredClassrooms = ctrl.classrooms.filter(classroom => {
                    return classroom.name.toLowerCase().includes(ctrl.search.toLowerCase());
                });
            } else {
                ctrl.filteredClassrooms = ctrl.classrooms;
            }

            ctrl.total = { actives: 0, not_actives: 0, not_involved: 0, total: 0 };
            ctrl.filteredClassrooms.forEach(classroom => {
              const participation = classroom.participation;
              ctrl.total.actives += participation.actives;
              ctrl.total.not_actives += participation.not_actives;
              ctrl.total.not_involved += participation.not_involved;
            });
            ctrl.total.total = ctrl.total.actives + ctrl.total.not_actives + ctrl.total.not_involved;
        }

        function toggleDetails() {
            ctrl.minimized = !ctrl.minimized;
        }
    }
})();
