(function(angular) {
    'use strict';
    /* Controllers */
    var app = angular.module('reports.controllers', []);

    app.controller('ReportsCtrl', ['$scope', '$location', '$sce', 'SetLanguage', '$window', 'CourseUserReport', 'LessonsUserProgress', 'BasicClass', 'CourseStats', 'CourseProfessor',
        function($scope, $location, $sce, SetLanguage, $window, CourseUserReport, LessonsUserProgress, BasicClass, CourseStats, CourseProfessor, ) {
            $scope.course_id = parseInt($window.course_id, 10);
            var current_user_id = parseInt($window.user_id, 10);
            
            $scope.user_reports_total_items = 0
            $scope.duplicatedCounter = 0
            $scope.ordering = '-name'
            $scope.reverse = false;
            $scope.my_classes = [];
            $scope.others_classes = [];
            $scope.reports_loaded = false;
            $scope.filters = {
                selected_class: "all",
                textsearch: ''
            }

            $scope.setLanguage = function(language) {
                SetLanguage(language);
                $window.location.reload();
                $scope.language = language;
            }

            // Pagination Controls
            $scope.pagination = {
                report_pages_max_number: 100,
                report_page_size: 50,
                currentPage: 1,
            }
            
            // Query all existing elements
            $scope.initial_reports = getSinglePageReport(1);
            
            function getSinglePageReport (pageToGet) {
                return CourseUserReport.get({
                    course: $scope.course_id,
                    page: pageToGet || $scope.pagination.currentPage,
                    search: $scope.filters.textsearch,
                },
                (data) => {
                    $scope.users_reports = data.results
                    $scope.user_reports_total_items = data.count
                    $scope.reports_loaded = true;
                    $scope.users_reports.$resolved = true
                })
            }

            $scope.course_stats = CourseStats.get({
                courseId: $scope.course_id
            });
            

            CourseProfessor.query({
                course: $scope.course_id,
                user: current_user_id
            }, function(course_professor) {
                var current_user = course_professor[0];
                var current_user_role = '';
                // If current_user is undefined, he is not course professor, but may be admin
                if (current_user === undefined) {
                    if ($window.is_admin)
                    // if user is admin, set role to coordinator, higher role in course.
                        current_user_role = 'coordinator';
                } else {
                    current_user_role = current_user.role;
                }

                $scope.classes = BasicClass.query({
                    course: $scope.course_id
                }, function(classes) {
                    if (current_user_role === 'assistant') {
                        $scope.my_classes = classes;
                        $scope.filters.selected_class = 'my_classes';
                    } else if (current_user_role === 'coordinator') {
                        $scope.filters.selected_class = 'all';
                        classes.forEach(function(klass) {
                            // if current user is undefined, he is not course professor, so he don't have any class
                            // in this course.
                            if (current_user !== undefined && klass.assistants.includes(current_user.user)) {
                                $scope.my_classes.unshift(klass);
                            } else {
                                $scope.others_classes.unshift(klass);
                            }
                        });
                    }
                });
                $scope.current_user_role = current_user_role;
            });

            $scope.show_user_progress_details = function(user) {
                if (user.lessons_stats === undefined) {
                    user.lessons_stats = LessonsUserProgress.get({
                        courseId: $scope.course_id,
                        user: user.user_id
                    });
                }
            };

            $scope.clean_keyword_search = () => {
                $scope.filters.textsearch = ''
                $scope.filter_stats()
            }
            

            $scope.filter_stats = function() {               
                if ($scope.filters.selected_class === 'all') {
                    $scope.course_stats = CourseStats.get({
                        courseId: $scope.course_id,
                    });
                    $scope.users_reports = CourseUserReport.get({
                        course: $scope.course_id,
                        page:  $scope.pagination.currentPage,
                        search: $scope.filters.textsearch
                    }, function(data) {
                        $scope.users_reports = data.results;
                        $scope.user_reports_total_items = data.count;
                        $scope.reports_loaded = true;
                    });
                } else if ($scope.filters.selected_class === 'my_classes') {
                    const sortedMyClasses = $scope.my_classes.map(function(x) {
                        return x.id;
                    })
                    $scope.users_reports = CourseUserReport.get({
                        course: $scope.course_id,
                        page:  $scope.pagination.currentPage,
                        classes: sortedMyClasses,
                        search: $scope.filters.textsearch,
                    }, (data) => {
                        $scope.users_reports = data.results;
                        $scope.user_reports_total_items = data.count;
                        $scope.reports_loaded = true;
                    });
                    $scope.course_stats = CourseStats.get({
                        courseId: $scope.course_id,
                        classes: sortedMyClasses
                    });
                } else if ($scope.filters.selected_class === 'others_classes') {
                    const sortedOtherClasses = $scope.others_classes.map(function(x) {
                        return x.id;
                    })
                    $scope.users_reports = CourseUserReport.get({
                        course: $scope.course_id,
                        classes: sortedOtherClasses,
                        page:  $scope.pagination.currentPage,
                        search: $scope.filters.textsearch,
                    }, (data) => {
                        $scope.users_reports = data.results;
                        $scope.user_reports_total_items = data.count;
                        $scope.reports_loaded = true;
                    });
                    $scope.course_stats = CourseStats.get({
                        courseId: $scope.course_id,
                        classes: sortedOtherClasses
                    });
                } else {
                    $scope.users_reports = CourseUserReport.get({
                        course: $scope.course_id,
                        classes: $scope.filters.selected_class,
                        page:  $scope.pagination.currentPage,
                        search: $scope.filters.textsearch,
                    }, (data) => {
                        $scope.users_reports = data.results;
                        $scope.user_reports_total_items = data.count;
                        $scope.reports_loaded = true;
                    });
                    $scope.course_stats = CourseStats.get({
                        courseId: $scope.course_id,
                        classes: $scope.filters.selected_class
                    });
                }
            };
        }
    ]);

    app.controller('GeneralReportsCtrl', ['$scope', '$location', '$sce', 'GeneralSummary', 'Contract',
        function($scope, $location, $sce, GeneralSummary, Contract) {
            $scope.general_data = GeneralSummary.get({}, function() {}, function(error) {
                // alert('Não é possível exportar relatórios agora. Por favor, converse com o suporte');
                // window.open('/dashboard', '_self');
            });
            $scope.contracts = Contract.query();

            // General report for dowload
            $scope.general_report = {};
            $scope.download_general_report = function() {
                var options = '?';
                for(var idx in $scope.general_report.groups) {
                    if(idx == 0)
                        options += ('group=' + $scope.general_report.groups[0].name);
                    else
                        options += (',' + $scope.general_report.groups[idx].name);
                }
                window.open('/legacy/api/users-by-group' + options + '&format=csv','_blank');
            };

            // Course report for download
            $scope.course_report = {};
            $scope.download_course_report = function() {
                var options = '?';
                for(var idx in $scope.course_report.classes) {
                    if(idx == 0)
                        options += ('id=' + $scope.course_report.classes[0].id);
                    else
                        options += (',' + $scope.course_report.classes[idx].id);
                }
                window.open('/legacy/api/users-by-class' + options + '&format=csv','_blank');
            }
        }
    ]);
})(angular);
