(function() {
    'use strict';

    angular
        .module('roleParticipationReport')
        .component('roleParticipationReport', {
            templateUrl: '/role-participation-report.template.html',
            controller: RoleParticipationReportCtrl,
            bindings: {
                date: '<',
                cities: '<',
                networks: '<',
                ntes: '<',
            },
        });

    RoleParticipationReportCtrl.$inject = ['$scope', 'RoleParticipationReport'];

    function RoleParticipationReportCtrl ($scope, RoleParticipationReport) {

        var ctrl = this;

        ctrl.city = '';
        ctrl.fetchRoles = throttleDebounce.debounce(1000, fetchRoles);
        ctrl.minimized = false;
        ctrl.network = '';
        ctrl.nte = '';
        ctrl.roles = null;
        ctrl.toggleDetails = toggleDetails;
        ctrl.total = null;

        ctrl.$onChanges = function() {
            fetchRoles();
        };

        function fetchRoles() {
            var params = { city: ctrl.city, date: ctrl.date, network: ctrl.network, nte: ctrl.nte };

            RoleParticipationReport.get(params, (data) => {
                ctrl.roles = data;
                ctrl.roles.$promise = undefined;
                ctrl.roles.$resolved = undefined;
                ctrl.total = { actives: 0, not_actives: 0, not_involved: 0, total: 0 };

                Object.values(ctrl.roles).forEach((role) => {
                  if (role !== undefined) {
                    ctrl.total.actives += role.actives;
                    ctrl.total.not_actives += role.not_actives;
                    ctrl.total.not_involved += role.not_involved;
                    ctrl.total.total += role.actives + role.not_actives + role.not_involved;
                  }
                });
            });
        }

        function toggleDetails() {
            ctrl.minimized = !ctrl.minimized;
        }
    }
})();
